%mobius_exploration
% Exploration of circle transformation ideas alluded to in Indra's Pearls
% by Mumford, Series and Wright.
%
% LAST UPDATED by Andy French July 2023

function mobius_exploration

%Define parameter which generates two kissing circles within another circle
k = 0.5;

%Define transformation matrices
a = [1,0; -2*1i, 1];
b = [1-1i, 1; 1, 1+1i ];

%Set up a blank figure window with a blank axes that can be added to with
%plot commands
axes('NextPlot','add'); axis equal; axis off;

%Plot three circles
circle( 1/2,0,1/2,[1,1,1] );
circle( k/2,0,k/2,[1,0,0] );
circle( k/2 + 1/2,0,(1-k)/2,[0,1,0] );

%Apply Mobius transformation
xc = 1/2; yc = 0; r = 1/2;
for n=1:50
    [xc, yc, r] = mobius( xc, yc, r, b );
    circle(xc, yc, r, rand(1,3) )
end
xc = k/2 + 1/2; yc = 0; r = (1-k)/2;
for n=1:50
    [xc, yc, r] = mobius( xc, yc, r, a );
    circle(xc, yc, r, rand(1,3) )
end

%Print figure and close
print( gcf,'mobius exploration.png','-dpng','-r300' ); close(gcf);

%%

%Determine transformation of a circle via a Mobius recipe. T is a
%2x2 transformation matrix [a,b; c,d]. From Indra's Pearls pp91.
function [xxc, yyc, rr] = mobius( xc, yc, r, T )
a = T(1,1); b = T(1,2); c = T(2,1); d = T(2,2);
P = xc + 1i*yc;
z = P - ( (r^2)/conj( d/c + P) );
Q = (a*z +b)./(c*z + d );
xxc = real(Q); yyc = imag(Q);
rr = abs( Q - ( a*(P + r) + b )./( c*(P + r) + d ) );

%%

%Plot a circle
function circle( xc,yc,r,faceRGB )
theta = linspace(0,2*pi,500);
x = xc + r*cos(theta); y = yc + r*sin(theta);
patch(x,y,faceRGB,'edgecolor',[0,0,0],'linewidth',0.5);

%End of code