%pappus_chain_mobius
% Plots a Pappus Chain construction - a ring of a ring of circles between
% two tangent circles investigated by Pappus of Alexandria in the 3rd
% century AD. This is really efficiently computed using a Mobius
% transformation!
%
% LAST UPDATED by Andy French July 2023

function pappus_chain_mobius

%% INPUTS %%

%Number of circles to plot in Pappus chain
N = 100;

%Define Pappus chain ratio ( 0 < a < 1)
phi = (1+sqrt(5))/2; a = 1/phi;
a = 5/7;

%

%Outer circle
[x1,y1] = circle( 1/2,0,1/2 );

%Inner circles
[x2,y2] = circle( a/2,0,a/2 );
[x3,y3] = circle( a/2 + 1/2,0,(1-a)/2 );

%Plot Pappus chain figure
axes('nextplot','add'); 
title(['Pappus chain : a = ',num2str(a)],'fontsize',14);
circleplot( x1,y1,[1,1,1] );
circleplot( x2,y2,rand(1,3) );
circleplot( x3,y3,rand(1,3) );
axis equal; axis off;

%Generate more circles using Mobius transform. This works by transforming
%a vertical stack of Inner circle coordinates [x3,y3]
for n=1:N
    [x,y] = circle( a/2 + 1/2 , 2*(n-1)*(1-a)/2, (1-a)/2 );
    [x,y] = mobius(x,y,a);
    circleplot( x,y,rand(1,3) );
    circleplot( x,-y,rand(1,3) );
end

%Save figure to a PNG file
filename = ['pappus a=',strrep( num2str(a,3),'.','p'),' mobius.png'];
print( gcf, filename,'-dpng','-r300' ); close(gcf);

%%

%Mobius transform
function [xx,yy] = mobius( x,y,a )
z = x + 1i*y;
z = a./z;
xx = real(z);  yy = imag(z);

%%

%Circle x,y coordinates
function [x,y] = circle( xc,yc,r )
theta = linspace(0,2*pi,500);
x = xc + r*cos(theta); y = yc + r*sin(theta);

%%

%Plot circle patch
function circleplot( x,y,rgb )
linewidth = 0.5; edgecolor = [0,0,0];
patch( x,y,rgb,'edgecolor',edgecolor,'linewidth',linewidth );

%End of code