%three_circles
% Solution to the problem of finding the centre coordinates and radius of a
% circle that touches three other circles.
% 
% https://math.stackexchange.com/questions/3100828/calculate-the-circle-that-touches-three-other-circles
%
% LAST UPDATED by Andy French July 2023

function three_circles

%Fontsize and linewidth
fsize = 18; linewidth = 1;

%Centre and radii of three circles
x_1 =-1; y_1 = 5; r_1 = 2;
x_2 =-2; y_2 = -1; r_2 = 2;
x_3 =3; y_3 = 2; r_3 = 3;

%%

%Solve for centre (x,y) and radius r of circles that touche three other circles
[x1,y1,r1,x2,y2,r2] = three_circle_touch( x_1,y_1,r_1,x_2,y_2,r_2,x_3,y_3,r_3 );

%Plot results
axes('nextplot','add','fontsize',fsize);
plot_circle( x_1,y_1,r_1,[1,0,0],NaN,linewidth );
plot_circle( x_2,y_2,r_2,[0,1,0],NaN,linewidth );
plot_circle( x_3,y_3,r_3,[0,0,1],NaN,linewidth );
plot_circle( x1,y1,r1,[0,0,0],NaN,linewidth );
plot_circle( x2,y2,r2,[0,0,0],NaN,linewidth );
axis equal; box on; grid on; grid minor; 
set(gca,'gridlinestyle','-','minorgridlinestyle','-');
title('Three circle intersection','fontsize',fsize);
print(gcf,'three_circles.png','-dpng','-r300');
close(gcf);

%%

%Solve for centre (x,y) and radius r of circle that touches three other circles
% Code from https://math.stackexchange.com/questions/3100828/calculate-the-circle-that-touches-three-other-circles
function [x1,y1,r1,x2,y2,r2] = three_circle_touch( x_1,y_1,r_1,x_2,y_2,r_2,x_3,y_3,r_3 )

% Find constant of circle #2- circle #1
K_a = -r_1^2+r_2^2+x_1^2-x_2^2+y_1^2-y_2^2;

% Find constant of circle #3- cirlce #1
K_b = -r_1^2+r_3^2+x_1^2-x_3^2+y_1^2-y_3^2;

% Find constants of [x=A_0+A_1*r, y=B_0+B_1*r]
D = x_1*(y_2-y_3)+x_2*(y_3-y_1)+x_3*(y_1-y_2);
A_0 =(K_a*(y_1-y_3)+K_b*(y_2-y_1))/(2*D);
B_0 = -(K_a*(x_1-x_3)+K_b*(x_2-x_1))/(2*D);
A_1 = -(r_1*(y_2-y_3)+r_2*(y_3-y_1)+r_3*(y_1-y_2))/D;
B_1 = (r_1*(x_2-x_3)+r_2*(x_3-x_1)+r_3*(x_1-x_2))/D;

% Find constants of C_0 + 2*C_1*r + C_2^2 = 0
C_0 = A_0^2-2*A_0*x_1+B_0^2-2*B_0*y_1-r_1^2+x_1^2+y_1^2;
C_1 = A_0*A_1-A_1*x_1+B_0*B_1-B_1*y_1-r_1;
C_2 = A_1^2+B_1^2-1;

% Solve for r,x,y
r1=( -sqrt(C_1^2-C_0*C_2)-C_1)/C_2;
x1 = A_0 + A_1*r1;
y1 = B_0 + B_1*r1;
r2=( sqrt(C_1^2-C_0*C_2)-C_1)/C_2;
x2 = A_0 + A_1*r2;
y2 = B_0 + B_1*r2;

%%

%Plot a circle of radius r centered on (x0,y0)
function c = plot_circle( x0,y0,r,edgecolor,facecolor,linewidth )
N = 1000;  theta = linspace(0,2*pi,N);
x = x0 + r*cos(theta);  y = y0 + r*sin(theta);
c = patch( x,y,facecolor,'edgecolor',edgecolor,'linewidth',linewidth );

%End of code